import Vue from 'vue'
// import m from './mutation-types'
import m from './mutation-types'

export default {
  [m.SESSID_UPDATE] (state, sessId) {
    state.sessId = sessId
  },
  [m.REGISTRY_UPDATE] (state, registry) {
    state.registry = registry
  },
  [m.DOCUMENT_UPDATE] (state, nodeData) {
    state.nodeData = nodeData
    // Rebuild cache
    for (let k in state.nodeData.nodes) {
      state.nodeIdxCache[state.nodeData.nodes[k].id] = k
    }
  },
  [m.ACTIVITY_UPDATE] (state, activity) {
    state.activity = activity
  },
  [m.NODE_RAISE] (state, nodes) {
    for (let k in nodes) {
      const sn = nodes[k]
      // Need to search by ID since its manipulating the list?
      let ni = state.nodeData.nodes.findIndex(n => n.id === sn.id)
      const node = state.nodeData.nodes[ni]
      state.nodeData.nodes.splice(ni, 1)
      state.nodeData.nodes.push(node) // put in last
    }
    // Rebuild cache
    //
    for (let k in state.nodeData.nodes) {
      state.nodeIdxCache[state.nodeData.nodes[k].id] = k
    }
  },
  [m.NODE_UPDATE] (state, nodes) {
    // If array
    for (let k in nodes) {
      const node = nodes[k]
      const ni = state.nodeIdxCache[node.id]
      if (ni === null) continue

      Object.assign(state.nodeData.nodes[ni], node)
      // Vue.set(state.nodeData.nodes, ni, node)
      if (node.id === state.nodeInspect.id) {
        // Update node inspect
        state.nodeInspect = node
      }
    }
  },
  [m.NODE_ADD] (state, nodes) {
    state.nodeData.nodes.push(...nodes)
    // Rebuild cache
    for (let k in state.nodeData.nodes) {
      state.nodeIdxCache[state.nodeData.nodes[k].id] = k
    }
  },
  [m.NODE_REMOVE] (state, nodes) {
    for (let k in nodes) {
      const node = nodes[k]
      // Need to search by ID since its manipulating the list
      const ni = state.nodeData.nodes.findIndex(n => n.id === node.id)
      state.nodeData.links = state.nodeData.links
        .filter(l => l.from !== node.id && l.to !== node.id)
      state.nodeData.triggers = state.nodeData.triggers
        .filter(l => l.from !== node.id && l.to !== node.id)
      state.nodeData.nodes.splice(ni, 1)
    }
    // Rebuild cache
    for (let k in state.nodeData.nodes) {
      state.nodeIdxCache[state.nodeData.nodes[k].id] = k
    }
  },
  [m.NODE_INSPECT] (state, nodeId) {
    const node = state.nodeData.nodes.find(n => n.id === nodeId)
    state.nodeInspect = node
  },
  [m.NODE_SELECTION_CLEAR] (state) {
    state.nodeSelection = {}
  },
  [m.NODE_SELECTION_ADD] (state, nodes) {
    for (let k in nodes) {
      const n = nodes[k]
      Vue.set(state.nodeSelection, n.id, n)
    }
  },
  [m.LINK_ADD] (state, link) {
    state.nodeData.links.push(link)
  },
  [m.LINK_REMOVE] (state, link) {
    const i = state.nodeData.links.findIndex(l => l === link)
    if (i === -1) return
    state.nodeData.links.splice(i, 1)
  },

  [m.TRIGGER_ADD] (state, trigger) {
    state.nodeData.triggers.push(trigger)
  },
  [m.TRIGGER_REMOVE] (state, trigger) {
    const i = state.nodeData.triggers.findIndex(l => l === trigger)
    if (i === -1) return
    state.nodeData.triggers.splice(i, 1)
  },
  [m.NOTIFICATION_ADD] (state, msg) {
    state.notifications.push(msg)
  },
  [m.NOTIFICATION_CLEAR] (state, msg) {
    state.notifications = []
  }
}
